@@ -37,6 +37,7 @@ def srr2link_md5(srr: str) -> zip:
         "fastq_bytes",
         "fastq_md5",
         "fastq_aspera",
+        "fastq_ftp",
     ]
     data = {
         "accession": srr,
@@ -49,25 +50,29 @@ def srr2link_md5(srr: str) -> zip:
     library_layout = srr_info["library_layout"]
     md5list = srr_info["fastq_md5"].split(";")
     ascp_links = srr_info["fastq_aspera"].split(";")
+    ftp_links = srr_info["fastq_ftp"].split(";")
 
     # filter out invalid links, which means not matched with library_layout
     valid_links = []
     valid_md5 = []
-    for md5, ascp_link in zip(md5list, ascp_links):
+    valid_ftp_links = []
+    for md5, ascp_link, ftp_link in zip(md5list, ascp_links, ftp_links):
         if library_layout == "PAIRED":
             if ascp_link.endswith("_1.fastq.gz") or ascp_link.endswith("_2.fastq.gz"):
                 valid_links.append(ascp_link)
                 valid_md5.append(md5)
+                valid_ftp_links.append(ftp_link)
         elif library_layout == "SINGLE":
             if ascp_link.endswith(".fastq.gz"):
                 valid_links.append(ascp_link)
                 valid_md5.append(md5)
+                valid_ftp_links.append(ftp_link)
         else:
             raise ValueError(f"library_layout {library_layout} is not supported")
 
-    valid_links = [f"era-fasq@{link}" for link in valid_links]
+    valid_links = [f"era-fasp@{link}" for link in valid_links]
 
-    return zip(valid_links, valid_md5)
+    return zip(valid_links, valid_ftp_links, valid_md5)
 
 
 def gsm2srx(gsm: str):
@@ -84,23 +89,42 @@ def gsm2srx(gsm: str):
     return srx_list
 
 
-def ascp_links_list2dict(ascp_links: list):
-    ascp_dict = {}
-    for link, md5 in ascp_links:
-        ascp_dict[link] = {"md5": md5}
-    return ascp_dict
+def ascp_links_list2dict(ascp_links: list, ascp=True) -> dict:
+    ascp_links_dict = {}
+    ftp_links_dict = {}
+    for link, ftp, md5 in ascp_links:
+        ascp_links_dict[link] = {"md5": md5}
+        ftp_links_dict[ftp] = {"md5": md5}
+    if ascp:
+        result_dict = ascp_links_dict
+    else:
+        result_dict = ftp_links_dict
+    return result_dict
 
 
-def srx2link_md5(srx: str):
+def ftp_links_list2dict(ftp_links: list):
+    ftp_dict = {}
+    for link, ftp, md5 in ftp_links:
+        ftp_dict[ftp] = {"md5": md5}
+    return ftp_dict
+
+
+def get_link_md5(accession: str, accessiontype, ascp=True):
     """
     first get srx related srr, then get related links
     """
-    srr_list = srx2srr(srx)
-    ascp_links = []
+    if accessiontype == "srx":
+        get_subacc = srx2srr
+    elif accessiontype == "gsm":
+        get_subacc = gsm2srx
+    else:
+        raise ValueError(f"accessiontype {accessiontype} is not supported")
+    srr_list = get_subacc(accession)
+    links = []
     for srr in srr_list:
-        ascp_links += srr2link_md5(srr)
-    ascp_dict = ascp_links_list2dict(ascp_links)
-    return ascp_dict
+        links += srr2link_md5(srr)
+    links_dict = ascp_links_list2dict(links, ascp)
+    return links_dict
 
 
 def gsm2link_md5(gsm: str):
@@ -111,10 +135,10 @@ def gsm2link_md5(gsm: str):
     if len(srx_list) != 1:
         ValueError(f"gsm {gsm} has {len(srx_list)} srx, excepted 1")
     srx = srx_list[0]
-    ascp_dict = srx2link_md5(srx)
+    ascp_dict = get_link_md5(srx, "GSM")
     return ascp_dict
 
 
 if __name__ == "__main__":
-    t1 = srx2link_md5("SRX764940")
+    t1 = get_link_md5("SRX764940")
     print(t1)
